package com.test;

import com.hyfinity.Namespaces;
import com.hyfinity.utils.FileUtils;
import com.hyfinity.utils.xml.DOMUtils;
import com.hyfinity.utils.xml.XDocument;
import com.hyfinity.xgate.XGatePlugin;
import java.io.FileOutputStream;
import java.util.Enumeration;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.fileupload.FileItem;
import org.w3c.dom.*;

/**
 * Example XGate plugin to handle file uploads.
 * This base64 encodes the file data, and inserts it into the XML message.
 */
public class FileUploadTestPlugin implements XGatePlugin
{

    /**
     * Process the request before it is passed to the platform.
     * @param input An XDocument object containing the current state of the XML message.
     *              The document this contains should be updated as required.
     * @param request The HttpServletRequest object that defines the request being processed.
     *              This provides access to any request or session parameters etc.
     * @param response The HttpServletResponse object that will eventually be written to.
     *              You should not write to this as that will be handled automatically,
     *              but you may want to add a cookie, or set a header for example.
     */
    public void processInput(XDocument input, HttpServletRequest request, HttpServletResponse response)
    {
        try
        {
            //check if this was a multipart request
            if (request instanceof FileUploadRequestWrapper)
            {
                FileUploadRequestWrapper furequest = (FileUploadRequestWrapper) request;

                //process through each file included in the request
                Enumeration fileEnum = furequest.getFileNames();
                while(fileEnum.hasMoreElements())
                {
                    String fileFieldName = (String) fileEnum.nextElement();
                    FileItem fi = furequest.getFile(fileFieldName);

                    if (fi.getSize() > 0)
                    {
                        //base64 encode the file contents and add to the XML
                        byte[] encoded = Base64.encodeBase64(fi.get());
                        Element elem = input.getDocument().createElementNS(Namespaces.XGATE, "xg:attribute");
                        elem.setAttributeNS(Namespaces.XGATE, "xg:name", fileFieldName);
                        DOMUtils.setText(elem, new String(encoded));
                        input.getRootNode().appendChild(elem);

                        //insert content type and filename elements in the XML

                        //content type
                        Element typeElem = input.getDocument().createElementNS(Namespaces.XGATE, "xg:attribute");
                        typeElem.setAttributeNS(Namespaces.XGATE, "xg:name", fileFieldName + "type");
                        DOMUtils.setText(typeElem, fi.getContentType());
                        input.getRootNode().appendChild(typeElem);
                        //filename
                        Element nameElem = input.getDocument().createElementNS(Namespaces.XGATE, "xg:attribute");
                        nameElem.setAttributeNS(Namespaces.XGATE, "xg:name", fileFieldName + "name");
                        DOMUtils.setText(nameElem, FileUtils.removeDirectory(fi.getName()));
                        input.getRootNode().appendChild(nameElem);

                        //If the file itself has a binding location defined, set up the content type and
                        //filename to be created as attributes
                        if (request.getParameter(fileFieldName + "_xpath") != null)
                        {
                            //insert content type binding
                            Element typeBindingElem = input.getDocument().createElementNS(Namespaces.XGATE, "xg:attribute");
                            typeBindingElem.setAttributeNS(Namespaces.XGATE, "xg:name", fileFieldName + "type_xpath");
                            DOMUtils.setText(typeBindingElem, request.getParameter(fileFieldName + "_xpath") + "/@type");
                            input.getRootNode().appendChild(typeBindingElem);

                            //insert filename binding
                            Element nameBindingElem = input.getDocument().createElementNS(Namespaces.XGATE, "xg:attribute");
                            nameBindingElem.setAttributeNS(Namespaces.XGATE, "xg:name", fileFieldName + "name_xpath");
                            DOMUtils.setText(nameBindingElem, request.getParameter(fileFieldName + "_xpath") + "/@filename");
                            input.getRootNode().appendChild(nameBindingElem);
                        }

                    }
                }
            }
        }
        catch (Exception e)
        {
            System.out.println("Error processing file! ");
            e.printStackTrace();
        }
    }


    /**
     * Process the response recieved from the platform, before it is returned to the browser.
     * @param output An XDocument object containing the current state of the XML message (this will often be HTML).
     *              The document this contains should be updated as required.
     * @param request The HttpServletRequest object that defines the request being processed.
     *              This provides access to any request pr seesion parameters etc.
     * @param response The HttpServletResponse object that will eventually be written to.
     *              You should not write to this as that will be handled automatically,
     *              but you may want to add a cookie, or set a header for example.
     */
    public void processOutput(XDocument output, HttpServletRequest request, HttpServletResponse response)
    {
        //check for any files that need to be temporarily saved out.
        try
        {
            //this assumes the data for each file is placed under a files element which is directly under the html element
            NodeList files = output.selectNodeList("/*/files/*");
            for (int i = 0; i < files.getLength(); ++i)
            {
                Element elem = (Element) files.item(i);

                String encoded = DOMUtils.getText(elem);
                if ((encoded.length() > 0) && (elem.getAttribute("filename") != null))
                {
                    //decode the base 64 information
                    byte[] decoded = Base64.decodeBase64(encoded.getBytes());

                    //write out to a file
                    //this currently uses the filename attribute, and places it in the root of the webapp
                    //in reality these files will probably want to be placed in a temp directory, and maybe use the session ID in the file names
                    //any changes here will also need to be made where the link is placed in the HTML.
                    //This may require placing extra parameters (eg session id) into the request message using the method above.
                    FileOutputStream fos = new FileOutputStream(request.getSession().getServletContext().getRealPath(elem.getAttribute("filename")));
                    fos.write(decoded);
                    fos.close();
                }
            }

            //remove the files element from the generated HTML returned.
            if (files.getLength() > 0)
            {
                Node toRemove = files.item(0).getParentNode();
                toRemove.getParentNode().removeChild(toRemove);
            }

        }
        catch (Exception e)
        {
            System.out.println("Error outputting file " + e.getMessage());
            e.printStackTrace();
        }



    }

}
